import unittest

from dici import dici
from robber import expect


class DiciTest(unittest.TestCase):

    def test_instantiate_dici_with_keyword_arguments(self):
        dictionary = dici(foo='bar', fizz='buzz')
        expect(dictionary.foo).to.eq('bar')
        expect(dictionary.fizz).to.eq('buzz')

    def test_handles_setting_fields_to_dictionary_values(self):
        dictionary = dici()
        dictionary.foo = {
            'bar': True
        }
        expect(dictionary.foo.bar).to.be.true()

    def test_compare_a_dici_with_a_dictionary(self):
        dictionary = dici(foo='bar', fizz='buzz')
        expect(dictionary == {
            'foo': 'bar',
            'fizz': 'buzz'
        }).to.be.true()

    def test_get_field_value_from_dici_with_dot_notation(elf):
        dictionary = dici()
        dictionary.foo = {
            'bar': 'baz',
            'fizz': 'buzz'
        }
        expect(dictionary.foo.bar).to.eq('baz')

    def test_get_field_value_from_dici_with_dot_notation_within_index_notation(self):
        dictionary = dici()
        dictionary.foo = {
            'bar': 'baz',
            'fizz': 'buzz'
        }
        expect(dictionary['foo.bar']).to.eq('baz')

    def test_add_field_to_dici_with_dot_notation(self):
        dictionary = dici()
        dictionary.foo = 'bar'
        expect(dictionary.foo).to.eq('bar')

    def test_add_field_to_dici_with_index_notation(self):
        dictionary = dici()
        dictionary['foo'] = 'bar'
        expect(dictionary.foo).to.eq('bar')

    def test_delete_field_to_dici_with_dot_notation_within_index_notation(self):
        dictionary = dici()
        dictionary.foo = {
            'bar': 'baz',
            'fizz': 'buzz'
        }
        del dictionary['foo.bar']
        expect(dictionary).to.eq({
            'foo': {
                'fizz': 'buzz' 
            }
        })

    def test_delete_field_from_dici_with_dot_notation(self):
        dictionary = dici(foo='bar', fizz='buzz')
        del dictionary.foo
        expect(dictionary).to.eq({
            'fizz': 'buzz'
        })

    def test_check_dici_contains_key_element_with_in_expression(self):
        dictionary = dici(foo='bar', fizz='buzz')
        expect('foo' in dictionary).to.be.true()
        expect('fizz' in dictionary).to.be.true()

    def test_can_iterate_over_the_keys_of_a_dici(self):
        dictionary = dici(foo='bar', fizz='buzz')
        results = []
        for key in dictionary:
            results.append(key)
        expect(results).to.eq(['foo', 'fizz'])

    def test_dici_set_for_regular_field(self):
        dictionary = dici()
        dictionary.set('foo', 'bar')
        expect(dictionary.foo).to.eq('bar')

    def test_dici_set_for_regular_nested_field(self):
        dictionary = dici()
        dictionary.set('foo.bar', 'buzz')
        expect(dictionary.foo.bar).to.eq('buzz')



"""
dici - dicts made easy
"""

class dici(dict):

    def __init__(self, **fields):
        for (key, value) in fields.items():
            self.__setitem__(key, value)

    def __delitem__(self, key):
        if '.' in key:
            obj = self
            prev = None
            for part in key.split('.'):
                prev = obj
                obj = obj[part]
            del prev[part]
        else: dict.__delitem__(self, key)

    __delattr__ = __delitem__

    def __setitem__(self, key, value):
        if isinstance(value, dict):
            value = dici(**value)
            dict.__setitem__(self, key, value)

        else:
            dict.__setitem__(self, key, value)

    def __getitem__(self, key):
        if '.' in key:
            obj = self

            for part in key.split('.'):
                if obj is None:
                    return None

                obj = obj[part]

            return obj
        else:
            return dict.__getitem__(self, key)

    def __getattr__(self, key):
        if key in self.__dict__:
            return dict.__getattr__(self, key)
        else:
            return self.__getitem__(key)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __contains__(self, item):
        try:
            self.__getitem__(item)
            return True
        except:
            return False

    def __iter__(self):
        return dict.__iter__(self)

    def set(self, key, value):
        """
        set the key by making sure to construct a path with separate
        dictionaries for each element in the key with a `.` separating
        the names in the key.
        """
        if '.' in key:
            obj = self
            for part in key.split('.')[:-1]:

                if part not in obj:
                    obj[part] = dici()

                obj = obj[part]

            exec('self.%s="%s"' % (key, value))

        else:
            dict.__setitem__(self, key, value)



"""
setup.py
"""
from setuptools import setup, find_packages

setup(
    name='dici',
    version='0.2.1',
    author='Rodney Gomes',
    author_email='rodneygomes@gmail.com',
    url='',
    install_requires=[],
    tests_require=[
        'robber==1.0.1'
    ],
    test_suite='test',
    keywords=[''],
    py_modules=['dici'],
    packages=find_packages(exclude=['test']),
    license='Apache 2.0 License',
    description='easy to use dict replacement',
    long_description='',
)
